package com.ruoyi.bst.device.utils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

import com.ruoyi.bst.device.constants.DeviceCacheKey;
import com.ruoyi.bst.device.domain.Device;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceLocationType;
import com.ruoyi.bst.orderDevice.domain.enums.OrderDeviceStatus;
import com.ruoyi.common.utils.JsonUtils;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.map.GpsCoordinateUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.IotDeviceSysInfo;
import com.ruoyi.iot.service.impl.DeviceOnlineStatus;

public class DeviceUtil {

    /**
     * 设置设备系统信息
     *
     * @param device          设备
     * @param sys             设备信息
     * @param at              时间
     * @param locationType    定位类型
     */
    public static void setIotSysInfo(DeviceVO device, IotDeviceSysInfo sys, LocalDateTime at, DeviceLocationType locationType) {
        if (device == null || sys == null) {
            return;
        }

        // 只有定位是正常的 && 卫星信号不弱 才认为是有获取到定位
        if (locationType == DeviceLocationType.DEVICE && DeviceUtil.validLocation(sys.getLon(), sys.getLat()) && !DeviceUtil.isLowSatelliteSignal(sys.getS())) {
            // 转换经纬度坐标系并赋值
            List<BigDecimal> coordinates = GpsCoordinateUtils.coordinateConvert(sys.getLon(), sys.getLat());

            if (coordinates != null && coordinates.size() >= 2) {
                device.setLongitude(coordinates.get(0));
                device.setLatitude(coordinates.get(1));
                device.setLastLocationTime(at);
            }
        }

        // 手机定位
        if (locationType == DeviceLocationType.PHONE && DeviceUtil.validMobileLocation(sys.getLon(), sys.getLat())) {
            device.setMobileLon(sys.getLon());
            device.setMobileLat(sys.getLat());
            device.setMobileLocationTime(at);
        }

        device.setLastTime(at);
        device.setVoltage(sys.getBat());
        device.setSignalStrength(sys.getCsq());
        device.setQuality(sys.getQ());
        device.setSatellites(sys.getS());
        device.setIotStatus(sys.getStatus());
    }

    /**
     * 设置设备信息
     *
     * @param device 设备
     * @param iot    设备信息
     */
    public static void setIotInfo(DeviceVO device, IotDeviceInfo iot) {
        if (device == null || iot == null) {
            return;
        }

        IotDeviceSysInfo sys = iot.getSys();
        if (sys != null) {
            setIotSysInfo(device, sys, iot.getAt(), DeviceLocationType.DEVICE);
        }
        device.setSoftwareVersion(iot.getVer());
    }

    /**
     * 设置设备信息
     *
     * @param device 设备
     * @param info   设备信息
     */
    public static void setIotInfo(Device device, Map<String, Object> info) {
        if (device == null || info == null) {
            return;
        }
        
        // 设置设备信息
        if (info.containsKey(DeviceCacheKey.VOLTAGE)) {
            device.setVoltage(JsonUtils.getBigDecimal(info.get(DeviceCacheKey.VOLTAGE)));
        }
        if (info.containsKey(DeviceCacheKey.SIGNAL_STRENGTH)) {
            device.setSignalStrength(JsonUtils.getInteger(info.get(DeviceCacheKey.SIGNAL_STRENGTH)));
        }
        if (info.containsKey(DeviceCacheKey.QUALITY)) {
            device.setQuality(String.valueOf(info.get(DeviceCacheKey.QUALITY)));
        }
        if (info.containsKey(DeviceCacheKey.SATELLITES)) {
            device.setSatellites(JsonUtils.getInteger(info.get(DeviceCacheKey.SATELLITES)));
        }
        if (info.containsKey(DeviceCacheKey.IOT_STATUS)) {
            device.setIotStatus(String.valueOf(info.get(DeviceCacheKey.IOT_STATUS)));
        }
        if (info.containsKey(DeviceCacheKey.LONGITUDE)) {
            device.setLongitude(JsonUtils.getBigDecimal(info.get(DeviceCacheKey.LONGITUDE)));
        }
        if (info.containsKey(DeviceCacheKey.LATITUDE)) {
            device.setLatitude(JsonUtils.getBigDecimal(info.get(DeviceCacheKey.LATITUDE)));
        }
        if (info.containsKey(DeviceCacheKey.LAST_TIME)) {
            device.setLastTime(JsonUtils.getLocalDatetime(info.get(DeviceCacheKey.LAST_TIME)));
        }
        if (info.containsKey(DeviceCacheKey.LAST_LOCATION_TIME)) {
            device.setLastLocationTime(JsonUtils.getLocalDatetime(info.get(DeviceCacheKey.LAST_LOCATION_TIME)));
        }
        if (info.containsKey(DeviceCacheKey.SOFTWARE_VERSION)) {
            device.setSoftwareVersion(String.valueOf(info.get(DeviceCacheKey.SOFTWARE_VERSION)));
        }

        // 手机定位
        if (info.containsKey(DeviceCacheKey.MOBILE_LON)) {
            device.setMobileLon(JsonUtils.getBigDecimal(info.get(DeviceCacheKey.MOBILE_LON)));
        }
        if (info.containsKey(DeviceCacheKey.MOBILE_LAT)) {
            device.setMobileLat(JsonUtils.getBigDecimal(info.get(DeviceCacheKey.MOBILE_LAT)));
        }
        if (info.containsKey(DeviceCacheKey.MOBILE_LOCATION_TIME)) {
            device.setMobileLocationTime(JsonUtils.getLocalDatetime(info.get(DeviceCacheKey.MOBILE_LOCATION_TIME)));
        }

        // 在线状态
        if (info.containsKey(DeviceCacheKey.ONLINE_STATUS)) {
            device.setOnlineStatus(String.valueOf(info.get(DeviceCacheKey.ONLINE_STATUS)));
        }
        if (info.containsKey(DeviceCacheKey.LAST_ONLINE_TIME)) {
            device.setLastOnlineTime(JsonUtils.getLocalDatetime(info.get(DeviceCacheKey.LAST_ONLINE_TIME)));
        }
    }

    /**
     * 判断设备是否有订单正在使用
     *
     * @param device 设备
     * @return 是否正在使用
     */
    public static boolean isOrderUsing(DeviceVO device) {
        return device != null && device.getOrderDeviceId() != null
                && OrderDeviceStatus.inUse().contains(device.getOrderDeviceStatus());
    }

    /**
     * 判断经纬度是否有效
     *
     * @param lon 经度
     * @param lat 纬度
     * @return 是否有效
     */
    public static boolean validLocation(BigDecimal lon, BigDecimal lat) {
        return lon != null && lat != null
                && !MathUtils.equals(lon, BigDecimal.ZERO)
                && !MathUtils.equals(lat, BigDecimal.ZERO);
    }

    /**
     * 判断设备信号是否弱
     *
     * @param device 设备
     * @return 是否弱
     */
    public static boolean isLowSignal(DeviceVO device) {
        return device != null && device.getSignalStrength() != null && device.getSignalStrength() < 10;
    }

    /**
     * 判断卫星信号是否弱
     *
     * @param device 设备
     * @return 是否弱
     */
    public static boolean isLowSatelliteSignal(DeviceVO device) {
        return device != null && isLowSatelliteSignal(device.getSatellites());
    }

    /**
     * 卫星信号弱
     * @param s
     * @return
     */
    public static boolean isLowSatelliteSignal(Integer s) {
        return s == null || s <= 5;
    }

    /**
     * 判断设备是否离线
     *
     * @param device 设备
     * @return 是否离线
     */
    public static boolean isOffline(DeviceVO device) {
        return device != null && device.getOnlineStatus() != null && DeviceOnlineStatus.OFFLINE.getStatus().equals(device.getOnlineStatus());
    }

    /**
     * 判断手机定位是否有效
     * 手机定位经纬度不能为0
     * 需要大于6位小数
     * @param lon
     * @param lat
     * @return
     */
    public static boolean validMobileLocation(BigDecimal lon, BigDecimal lat) {
        return lon != null && lat != null 
            && !MathUtils.equals(lon, BigDecimal.ZERO) && !MathUtils.equals(lat, BigDecimal.ZERO)
            && lon.scale() >= 6 && lat.scale() >= 6;
    }
}
